#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT 2
#define VENDOR_RQ_GET_PUSH 3
#define VENDOR_DEBUG 4
#define GET_LIGHT_B1 5
#define GET_LIGHT_B2 6
#define GET_LIGHT_B3 7
/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

char in_c(char pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);    
} 
char in_b(char pin)
{
    // read logic level of port-B's pin
    DDRB &= ~(1<<pin);
    return ((PINB & (1<<pin))>>pin);    
} 
void out_b(char pin, char val)
{
    // write logic level to port-B's pin
    DDRB |= 1<<pin;
    if(val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}
void out_c(char pin, char val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}
uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000;
    ADMUX |= channel;
    ADCSRA = 0b11000110;
    while ((ADCSRA & (1<<ADSC)));
    return ADCL + ADCH*256;
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */
    uint16_t min =999,tmp=0;
    int i;
    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if(rq->bRequest == VENDOR_DEBUG)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_b(led_no,led_val);
	return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if(rq->bRequest == VENDOR_RQ_GET_LIGHT)
    {
        ADMUX = 0b01000000;
        ADMUX |= 4;
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC)));
        dataBuffer[0] = ADCL;
        dataBuffer[1] = ADCH;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    else if(rq->bRequest == GET_LIGHT_B1)
    {
        ADMUX = 0b01000000;
        ADMUX |= 2;
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC)));
        dataBuffer[0] = ADCL;
        dataBuffer[1] = ADCH;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    else if(rq->bRequest == GET_LIGHT_B2)
    {
        ADMUX = 0b01000000;
        ADMUX |= 5;
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC)));
        dataBuffer[0] = ADCL;
        dataBuffer[1] = ADCH;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    else if(rq->bRequest == GET_LIGHT_B3)
    {
        ADMUX = 0b01000000;
        ADMUX |= 4;
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC)));
        dataBuffer[0] = ADCL;
        dataBuffer[1] = ADCH;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    else if(rq->bRequest == VENDOR_RQ_GET_PUSH)
    {
        i=0;
        out_c(0,1);
        while(i<1000)
        {
            tmp = read_adc(4);
            if(tmp<min)
                min = tmp;
            i++;
        }
        if(min < 400)
        {
            dataBuffer[0]=1;
            out_c(0,0);
        }
        else
            dataBuffer[0]=0;
        usbMsgPtr = dataBuffer;
        return 1;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = 0b00001011;  // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = 0b00000000;  // PC7..PC0 = 0
//    PORTC |= (1 << 3);   // Enable pull-up at PC3
    DDRD  = 0b00001000;
    PORTD = 0b00000000;

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
